from pathlib import Path
from typing import Any, Dict

from eth_utils.toolz import assoc, keyfilter
from ruamel.yaml import YAML

from eth2.beacon.helpers import compute_epoch_of_slot
from eth2.configs import Eth2Config


def generate_config_by_dict(dict_config: Dict[str, Any]) -> Eth2Config:
    filtered_keys = ("DOMAIN_", "EARLY_DERIVED_SECRET_PENALTY_MAX_FUTURE_EPOCHS")

    return Eth2Config(
        **assoc(
            keyfilter(
                lambda name: all(key not in name for key in filtered_keys), dict_config
            ),
            "GENESIS_EPOCH",
            compute_epoch_of_slot(
                dict_config["GENESIS_SLOT"], dict_config["SLOTS_PER_EPOCH"]
            ),
        )
    )


def load_yaml_at(p: Path) -> Dict[str, Any]:
    y = YAML(typ="unsafe")
    return y.load(p)


config_cache: Dict[Path, Eth2Config] = {}


def load_config_at_path(p: Path) -> Eth2Config:
    if p in config_cache:
        return config_cache[p]

    config_data = load_yaml_at(p)
    config = generate_config_by_dict(config_data)
    config_cache[p] = config
    return config
